package com.aselimov.app.steady_state_analysis;

import com.aselimov.app.timedata.TimeData;
import com.aselimov.app.timedata.TimeData.TimeDataPoints;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;

public class LinearRegressionTest {
    private LinearRegression linearRegression;
    private TimeData timeData;

    @BeforeEach
    void setUp() {
        // Set up a LinearRegression instance with reasonable default values
        linearRegression = new LinearRegression(0.01, 3, 5); // slope tolerance = 0.01, need 3 matches, window size = 5
        timeData = new TimeData();
    }

    @Test
    void testNotEnoughData() {
        // Test when there's not enough data points
        timeData.accept(new TimeData.TimeDataPoint(1.0, 1.0));
        timeData.accept(new TimeData.TimeDataPoint(2.0, 2.0));

        assertEquals(SteadyStateCalculator.SignalStatus.NOT_CHECKED, linearRegression.isSteady(timeData));
    }

    @Test
    void testSteadyStateDetection() {
        // Test steady state detection with a flat line
        for (int i = 0; i < 10; i++) {
            timeData.accept(new TimeData.TimeDataPoint(i * 1.0, 1.0)); // Constant value of 1.0
        }

        assertEquals(SteadyStateCalculator.SignalStatus.STEADY, linearRegression.isSteady(timeData));
    }

    @Test
    void testNonSteadyState() {
        // Test non-steady state with a linearly increasing signal
        for (int i = 0; i < 10; i++) {
            timeData.accept(new TimeData.TimeDataPoint(i * 1.0, i * 1.0)); // Linear increase
        }

        assertEquals(SteadyStateCalculator.SignalStatus.NOT_STEADY, linearRegression.isSteady(timeData));
    }

    @Test
    void testMultipleWindows() {
        // Test multiple windows with a signal that becomes steady
        // First add non-steady data
        for (int i = 0; i < 5; i++) {
            timeData.accept(new TimeData.TimeDataPoint(i * 1.0, i * 1.0)); // Linear increase
        }

        // Add steady data
        for (int i = 0; i < 10; i++) {
            timeData.accept(new TimeData.TimeDataPoint(i * 1.0 + 5.0, 5.0)); // Constant value of 5.0
        }

        assertEquals(SteadyStateCalculator.SignalStatus.STEADY, linearRegression.isSteady(timeData));
    }

    @Test
    void testSlopeTolerance() {
        // Test with a signal that has a small slope within tolerance
        linearRegression = new LinearRegression(0.1, 3, 5); // Increase slope tolerance

        for (int i = 0; i < 10; i++) {
            // Create a signal with a small slope (0.05)
            timeData.accept(new TimeData.TimeDataPoint(i * 1.0, 1.0 + i * 0.05));
        }

        assertEquals(SteadyStateCalculator.SignalStatus.STEADY, linearRegression.isSteady(timeData));
    }
}
